#ifndef CFILE_H

#define CFILE_H

#include <stdio.h>
#include <string>
#include <stdarg.h>

// piccola classe per gestire i file con le API C x renderli + amichevoli con le exception

class CFile
{
    FILE *file_;
    
    public:
        enum Mode {Read, Write, Append};
        enum SeekPos {Start = SEEK_SET, Current = SEEK_CUR, End = SEEK_END};
        CFile() : file_(NULL) {}
        CFile(const std::string &name, Mode mode = Read) : file_(NULL) { open(name, mode); }
        ~CFile() { close(); }

        bool good() const { return file_ != NULL; }

        bool open(const std::string &name, Mode mode = Read) {
            if (file_) close();
            const char *m;

            if (mode == Write)
                m = "wb";
            else if (mode == Append)
                m = "a";
            else
                m = "rb";
            
            return (file_ = ::fopen(name.c_str(), m)) != NULL;
        }
        void close() { if (file_) ::fclose(file_); file_ = NULL; }
      
        bool read(int &dest) {
            int d;

            if (file_ && fscanf(file_, "%d", &d) == 1) {
                dest = d;
                return true;
            }
            return false;
        }
        bool read(std::string &dest) {
            if (!file_)
                return false;

            dest.clear();

            char buffer[4096];
            while (!feof(file_)) {
                int len = fread(buffer, 1, sizeof(buffer), file_);
                if (len > 0)
                    dest.append(buffer, len);
                else if(!feof(file_))
                    return false;
            }
            return true;
        }
        size_t read(void *dest, size_t len) { 
            if (file_)
                return ::fread(dest, 1, len, file_); 
            else
                return 0;
        }
        size_t write(const void *source, size_t len) { 
            if (file_)
                return ::fwrite(source, 1, len, file_); 
            else
                return 0;
        }

        bool write(const std::string &source) { 
            if (file_)
                return ::fwrite(source.c_str(), 1, source.length(), file_) == source.length(); 
            else
                return false;
        } 
        int seek(size_t bytes, SeekPos position = Current) {
            if (file_)
                ::fseek(file_, bytes, position);
            else
                return -1;
        }
        bool writeln(const std::string &line) {
            if (!file_) return false;
            
            if (fwrite(line.c_str(), 1, line.length(), file_) != line.length())
                return false;

            fputc('\n', file_);

            return true;
        }
        bool put(int c) {
            if (!file_) return false;
            return fputc(c, file_) != EOF;
        }
        
        bool eof() const { if (file_) return feof(file_); else return true; }
       
        int vwritef(const char *format, va_list va) {
            if (!file_)
                return 0;

            return ::vfprintf(file_, format, va);
        }

        int writef(const char *format, ...) {
            if (!file_)
                return 0;

            va_list va;
            va_start(va, format);
            int rc = ::vfprintf(file_, format, va);
            va_end(va);

            return rc;
        }

        bool getline(void *buffer, size_t len) {
            if (file_)
                return ::fgets((char *)buffer, len, file_) != NULL;
            else
                return false;
        }
        long position() const {
            if (file_)
                return ::ftell(file_);
            else
                return -1;
        }
};

#include <dirent.h>

#ifdef WIN32
// su windows readdir e' rientrante
inline int readdir_r(DIR *d, dirent *ent, dirent **res) {
    if ((*res = readdir(d)))
        return 0;
    else
        return -1;
}
#endif

class CDir
{
    DIR *dir_;
public:
    CDir() : dir_(NULL) {}
    CDir(const std::string &path) { open(path); }
    ~CDir() { close(); }
    
    bool good() const { return dir_ != NULL; }
    bool open(const std::string &path) { return (dir_ = ::opendir(path.c_str())) != NULL; }
    void close() { if (dir_) ::closedir(dir_); dir_ = NULL; }

    bool get(std::string &name) {
        dirent *e = NULL;
        char buffer[sizeof(struct dirent) + 256];
        if ((::readdir_r(dir_, (dirent *) buffer, &e) == 0) && e) {
            // skippo i file "." e ".."
            if (e->d_name[0] == '.' &&
                (e->d_name[1] == 0 || (e->d_name[1] == '.' && e->d_name[2] == 0))
                )
                return get(name);
            
            name = e->d_name;
            return true;
        }
        return false;
    }
};

#endif
